@@ -37,12 +37,14 @@ class Dos:
             spin='up',
             soc_axis=None,
             combination_method="add",
+            sp_method='percentage',
             shift_efermi=0,
     ):
         self.folder = folder
         self.spin = spin
         self.soc_axis = soc_axis
         self.combination_method = combination_method
+        self.sp_method = sp_method
         self.incar = Incar.from_file(
             os.path.join(folder, 'INCAR')
         )
@@ -162,14 +164,34 @@ class Dos:
 
 
     def _check_f_orb(self):
+        f_elements = [
+            'La',
+            'Ac',
+            'Ce',
+            'Tb',
+            'Th',
+            'Pr',
+            'Dy',
+            'Pa',
+            'Nd',
+            'Ho',
+            'U',
+            'Pm',
+            'Er',
+            'Np',
+            'Sm',
+            'Tm',
+            'Pu',
+            'Eu',
+            'Yb',
+            'Am',
+            'Gd',
+            'Lu',
+        ]
         f = False
         for element in self.poscar.site_symbols:
-            if element != 'H':
-                E = Element(element)
-                orbitals = list(E.atomic_orbitals.keys())
-                for orb in orbitals:
-                    if 'f' in orb:
-                        f = True
+            if element in f_elements:
+                f = True
         
         return f
 
@@ -258,7 +280,10 @@ class Dos:
             if self.combination_method == "add":
                 tdos = np.c_[tdos[:,0], tdos_up + tdos_down]
             if self.combination_method == "sub":
-                tdos = np.c_[tdos[:,0], tdos_up - tdos_down]
+                if self.sp_method == 'percentage':
+                    tdos = np.c_[tdos[:,0], (tdos_up - tdos_down) / (tdos_up + tdos_down)]
+                elif self.sp_method == 'absolute':
+                    tdos = np.c_[tdos[:,0], tdos_up - tdos_down]
 
         return tdos
 
@@ -400,7 +425,10 @@ class Dos:
             if self.combination_method == 'add':
                 pdos = pdos_up + pdos_down
             if self.combination_method == 'sub':
-                pdos = pdos_up - pdos_down
+                if self.sp_method == 'percentage':
+                    pdos = (pdos_up - pdos_down) / (pdos_up + pdos_down)
+                elif self.sp_method == 'absolute':
+                    pdos = pdos_up - pdos_down
 
         #  tdos = self.tdos_array[:,-1]
         #  summed_pdos = np.sum(np.sum(pdos, axis=1), axis=1)
@@ -704,7 +732,7 @@ class Dos:
 
         return summed_layers
 
-    def _add_legend(self, ax, names, colors, fontsize=5, markersize=2):
+    def _add_legend(self, ax, names, colors, fontsize=10, markersize=4):
         legend_lines = []
         legend_labels = []
         for name, color in zip(names, colors):
@@ -751,6 +779,17 @@ class Dos:
         energy = energy[energy_in_plot_index]
         projected_data = projected_data[energy_in_plot_index]
 
+        unique_colors = np.unique(colors)
+
+        if len(unique_colors) == len(colors):
+            pass
+        else:
+            unique_inds = [np.isin(colors, c) for c in unique_colors]
+            projected_data = np.c_[
+                [np.sum(projected_data[:,i], axis=1) for i in unique_inds]
+            ].transpose()
+            colors = unique_colors
+
         if sigma > 0:
             for i in range(projected_data.shape[-1]):
                 projected_data[:,i] = self._smear(
@@ -989,7 +1028,7 @@ class Dos:
                     alpha=alpha,
                 )
 
-    def plot_spd(self, ax, orbitals='spd', fill=True, alpha=0.3, alpha_line=1.0, linewidth=1.5, sigma=0.05, energyaxis='y', color_dict=None, legend=True, total=True, erange=[-6, 6]):
+    def plot_spd(self, ax, orbitals='spd', fill=True, alpha=0.3, alpha_line=1.0, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6]):
         """
         This function plots the total density of states with the projected
         density of states for the total projections of the s, p, and d orbitals.
@@ -1014,15 +1053,18 @@ class Dos:
 
         projected_data = self._sum_spd(spd=orbitals)
 
-        if color_dict is None:
-            color_dict = {
-                0: self.color_dict[0],
-                1: self.color_dict[1],
-                2: self.color_dict[2],
-                3: self.color_dict[4],
-            }
-
-        colors = np.array([color_dict[self.spd_relations[i]] for i in orbitals])
+        if color_list is None:
+            color_list = [
+                self.color_dict[0],
+                self.color_dict[1],
+                self.color_dict[2],
+                self.color_dict[4]
+            ]
+            colors = np.array(
+                [color_list[i] for i in range(len(orbitals))]
+            )
+        else:
+            colors = color_list
 
         self._plot_projected_general(
             ax=ax,
@@ -1464,10 +1506,13 @@ class Dos:
         contour=False,
         levels=10,
         min_cutoff=1e-7,
+        max_cutoff=None,
         atol=None,
         custom_layer_inds=None,
         custom_cbar_label=None,
         cbar_orientation='vertical',
+        show_bounds=False,
+        set_bounds=None,
     ):
         """
         This function plots a layer by layer heat map of the density
@@ -1558,10 +1603,20 @@ class Dos:
                 min_val = np.min(densities)
                 if min_val < min_cutoff:
                     min_val = min_cutoff
-            norm = colors.LogNorm(vmin=min_val, vmax=np.max(densities))
+
+            if max_cutoff is not None:
+                max_val = max_cutoff
+            else:
+                max_val = np.max(densities)
+                
+            norm = colors.LogNorm(vmin=min_val, vmax=max_val)
         else:
             if self.combination_method == "sub" and self.spin == "both":
-                norm_val = np.max(np.abs([np.min(densities), np.max(densities)]))
+                if set_bounds is None:
+                    norm_val = np.max(np.abs([np.min(densities), np.max(densities)]))
+                else:
+                    norm_val = set_bounds
+
                 norm = colors.Normalize(vmin=-norm_val, vmax=norm_val)
             else:
                 norm = colors.Normalize(vmin=np.min(densities), vmax=np.max(densities))
@@ -1660,7 +1715,8 @@ class Dos:
                 min_val = im.norm.vmin
                 max_val = im.norm.vmax
                 cbar.set_ticks([min_val, max_val])
-                cbar.set_ticklabels(['Down', 'Up'])
+                if not show_bounds:
+                    cbar.set_ticklabels(['Down', 'Up'])
             else:
                 cbar.set_label('Density of States', fontsize=fontsize)
         else:
@@ -1679,8 +1735,15 @@ class Dos:
 
 
 if __name__ == '__main__':
-    dos = Dos(folder='../../vaspvis_data/slabdos')
+    dos = Dos(folder='../../vaspvis_data/dos_InAs')
     #  dos._sum_layers(layers=[0,1,2,3])
     fig, ax = plt.subplots(figsize=(4,3), dpi=100)
-    dos.plot_ldos(ax=ax, layers=range(10), erange=[-2,2], fill=False)
+    dos.plot_spd(
+        ax=ax,
+        erange=[-6,6],
+        fill=True,
+        total=False,
+        energyaxis='x',
+        #  color_list=['red', 'red', 'green'],
+    )
     plt.show()
